///////////////////////////////////////////////////////////////////
// ASAutoRun.cpp
// 720020-1163x
//
// Applied Data Systems
//
// This simple application demonstrates how to automatically run
// the device-side piece of ActiveSync (repllog.exe) when a serial
// cable is connected to the device.
//
// The serial port used must support the DCD (RLSD) line.  This example
// was written for and tested on the BitsyX, but should work on most ADS
// systems
//
//
// History
// -------
// 720020-11631: Nov 7, 2003 (ctacke)
//  - Initial implementation
///////////////////////////////////////////////////////////////////

#include "windows.h"

extern "C" BOOL CeEventHasOccurred (LONG lWhichEvent, TCHAR *pwszEndOfCommandLine);
#define NOTIFICATION_EVENT_RS232_DETECTED       9

int WINAPI WinMain(	HINSTANCE hInstance,
					HINSTANCE hPrevInstance,
					LPTSTR    lpCmdLine,
					int       nCmdShow)
{
	HANDLE	hPort	= NULL;
	DWORD	mask	= 0;
	HKEY	key;
	TCHAR	value[MAX_PATH];
	DWORD	status	= 0;

	// open the port
	hPort = CreateFile(_T("COM1:"), GENERIC_READ, 0, NULL, OPEN_EXISTING, 0, NULL);

	if(hPort != INVALID_HANDLE_VALUE)
	{
		// watch the DCD line
		SetCommMask(hPort, EV_RLSD);

		DEBUGMSG(TRUE, (_T("\r\nWaiting for connection...")));

		while((status & MS_RLSD_ON) == 0)
		{
			// wait
			WaitCommEvent(hPort, &mask, NULL);

			// did it transition to HIGH?
			// we only want to continue on a connect, not a disconnect
			GetCommModemStatus(hPort, &status);
		}

		if(hPort != INVALID_HANDLE_VALUE)
		{
			// close the port so ActiveSync can use it
			CloseHandle(hPort);

			// cable connected
			DEBUGMSG(TRUE, (_T("Connected!\r\n")));

			// make sure the right connection is selected in the registry
			int r = RegOpenKeyEx(HKEY_CURRENT_USER, _T("\\ControlPanel\\Comm"), 0, 0, &key);

			if(r != ERROR_SUCCESS)
			{
				DEBUGMSG(TRUE, (_T("Error! %i\r\n"), r));
				return 0;
			}
			_tcscpy(value, _T("19200@com1"));
			r = RegSetValueEx(key, _T("Cnct"), 0, REG_SZ, (BYTE*)value, (_tcslen(value) + 1) * sizeof(TCHAR));
			if(r != ERROR_SUCCESS)
			{
				DEBUGMSG(TRUE, (_T("Error! %i\r\n"), r));
				RegCloseKey(key);
				return 0;

			}
			RegCloseKey(key);

			// this is the way the driver sets the notification
			CeEventHasOccurred(NOTIFICATION_EVENT_RS232_DETECTED, NULL);

			// this would achieve a similar effect
			// CreateProcess(_T("repllog.exe"), NULL, NULL, NULL, FALSE, 0, NULL, NULL, NULL, NULL);
		}
	}

	return 0;
}
